package com.huimv.farm.damsubsidy.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.farm.damsubsidy.common.token.TokenSign;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.entity.BillBuyInfo;
import com.huimv.farm.damsubsidy.entity.BillIsolated;
import com.huimv.farm.damsubsidy.entity.BillLandingInspection;
import com.huimv.farm.damsubsidy.mapper.BillBuyInfoMapper;
import com.huimv.farm.damsubsidy.mapper.BillIsolatedMapper;
import com.huimv.farm.damsubsidy.entity.SysUser;
import com.huimv.farm.damsubsidy.mapper.BillLandingInspectionMapper;
import com.huimv.farm.damsubsidy.mapper.SysUserMapper;
import com.huimv.farm.damsubsidy.service.IBillLandingInspectionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 落地检 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-04-26
 */
@Service
public class BillLandingInspectionServiceImpl extends ServiceImpl<BillLandingInspectionMapper, BillLandingInspection> implements IBillLandingInspectionService {

    @Autowired
    private BillLandingInspectionMapper billLandingInspectionMapper;
    @Autowired
    private BillIsolatedMapper isolatedMapper;
    @Autowired
    private BillBuyInfoMapper buyInfoMapper;

    @Override
    public Result list(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");//信息类型
        String date = paramsMap.get("date");
        String startTime = paramsMap.get("startTime");
        String endTime = paramsMap.get("endTime");


        if (startTime != null) {
            startTime = startTime + " 00:00:00";
        }
        if (endTime != null) {
            endTime = endTime + " 23:59:59";
        }


        JSONArray jsonArray = new JSONArray();
        if (type == "1") { //检疫申请
            QueryWrapper<BillLandingInspection> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_time").between("create_time", startTime, endTime);
            List<BillLandingInspection> billLandingInspections = billLandingInspectionMapper.selectList(queryWrapper);
            for (BillLandingInspection billLandingInspection : billLandingInspections) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("name", billLandingInspection.getInspectionName());
                jsonObject.put("id", billLandingInspection.getId());
                jsonObject.put("time", billLandingInspection.getCreateTime());
                jsonObject.put("state", billLandingInspection.getInspectionSch());
                jsonArray.add(jsonObject);
            }
            return new Result(ResultCode.SUCCESS, jsonArray);
        }
        if (type == "2") { //隔离申请
            QueryWrapper<BillIsolated> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("str_isolated").between("str_isolated", startTime, endTime);
            List<BillIsolated> billIsolateds = isolatedMapper.selectList(queryWrapper);
            for (BillIsolated billIsolated : billIsolateds) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("name", billIsolated.getInspectionName());
                jsonObject.put("time", billIsolated.getStrIsolated());
                jsonObject.put("id", billIsolated.getId());
                jsonObject.put("state", billIsolated.getIsolatedSch());
                jsonArray.add(jsonObject);
            }
            return new Result(ResultCode.SUCCESS, jsonArray);
        }
        //交易信息
        if (type == "3") {
            QueryWrapper<BillBuyInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_time").between("create_time", startTime, endTime);
            List<BillBuyInfo> billBuyInfos = buyInfoMapper.selectList(queryWrapper);
            for (BillBuyInfo billBuyInfo : billBuyInfos) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("name", billBuyInfo.getBuyName());
                jsonObject.put("id", billBuyInfo.getId());
                jsonObject.put("time", billBuyInfo.getCreateTime());
                jsonArray.add(jsonObject);
            }
            return new Result(ResultCode.SUCCESS, jsonArray);
        }
        //补贴申请
        if (type == "4") {
            QueryWrapper<BillBuyInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.between("create_time", startTime, endTime).orderByDesc("create_time");
        }

        return null;
    }

    @Override
    public Result addApplication(HttpServletRequest httpServletRequest, BillLandingInspection billLandingInspection) {
        QueryWrapper<BillLandingInspection> landingInspectionQueryWrapper = new QueryWrapper<>();
        landingInspectionQueryWrapper.eq("inspection_num",billLandingInspection.getInspectionNum()).ne("inspection_sch",5);
        if (this.count(landingInspectionQueryWrapper) != 0){
            return new Result(10001,"存在相同检疫证号！",false);
        }
        billLandingInspection.setInspectionName(TokenSign.getUserName(httpServletRequest)+"的落地检疫申请");
        billLandingInspection.setCreateTime(DateTime.now());
        billLandingInspection.setCreateUser(Long.valueOf(TokenSign.getUserName(httpServletRequest)));
        billLandingInspection.setInspectionType(0);
        if (this.save(billLandingInspection)){
            return Result.SUCCESS();
        }else {
            return Result.FAIL();
        }
    }

    @Override
    public Result cancelApplication(HttpServletRequest httpServletRequest, BillLandingInspection billLandingInspection) {
        BillLandingInspection landingInspection = this.getById(billLandingInspection);
        landingInspection.setInspectionSch(5);
        if (this.updateById(landingInspection)) {
            return Result.SUCCESS();
        }else {
            return Result.FAIL();
        }
    }

    @Override
    public Result listApplication(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String sort = paramsMap.get("sort");
        String applicationStatus = paramsMap.get("applicationStatus");
        QueryWrapper<BillLandingInspection> landingInspectionQueryWrapper = new QueryWrapper<>();
        if (ObjectUtil.isNotEmpty(applicationStatus)){
            landingInspectionQueryWrapper.eq("inspection_sch",applicationStatus);
        }else {
            landingInspectionQueryWrapper.ne("inspection_sch",5);
        }
        if (sort.equals("1")){
            landingInspectionQueryWrapper.orderByDesc("create_time");
        }else {
            landingInspectionQueryWrapper.orderByAsc("create_time");
        }
        return null;
    }
}
