package com.huimv.farm.damsubsidy.service.impl;

import cn.hutool.core.date.DateTime;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.farm.damsubsidy.common.token.TokenSign;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.entity.BaseAnimal;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.entity.BillSubsidy;
import com.huimv.farm.damsubsidy.mapper.BaseAnimalMapper;
import com.huimv.farm.damsubsidy.mapper.BillSubsidyMapper;
import com.huimv.farm.damsubsidy.service.IBillSubsidyService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 补贴信息表 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-04-26
 */
@Service
public class BillSubsidyServiceImpl extends ServiceImpl<BillSubsidyMapper, BillSubsidy> implements IBillSubsidyService {

    @Autowired
    private BillSubsidyMapper subsidyMapper;

    @Override
    public Result list(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        Integer subsidySch = Integer.valueOf(paramsMap.get("subsidySch"));
        String startTime = paramsMap.get("startTime");
        String endTime = paramsMap.get("endTime");


        if (startTime != null) {
            startTime = startTime + " 00:00:00";
        }
        if (endTime != null) {
            endTime = endTime + " 23:59:59";
        }

        //从近到远
        QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();

            if (subsidySch == 7) { //全部进度
                queryWrapper.orderByDesc("sub_time").between("sub_time",startTime,endTime);
            } else {
                queryWrapper.eq("subsidy_sch", subsidySch).orderByDesc("sub_time")
                        .between("sub_time",startTime,endTime);
            }
            queryWrapper.eq("phone", paramsMap.get("phone"));
            List<BillSubsidy> billSubsidies = subsidyMapper.selectList(queryWrapper);
            JSONArray jsonArray = new JSONArray();
            for (BillSubsidy billSubsidy : billSubsidies) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("name", billSubsidy.getSubsidyName());
                jsonObject.put("date", billSubsidy.getSubTime());
                jsonObject.put("state", billSubsidy.getSubsidySch());
                jsonObject.put("id", billSubsidy.getId());
                jsonArray.add(jsonObject);
            }
            return new Result(ResultCode.SUCCESS, jsonArray);
    }

    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", paramsMap.get("id"));
        BillSubsidy billSubsidy = subsidyMapper.selectOne(queryWrapper);
        return new Result(ResultCode.SUCCESS, billSubsidy);
    }

    @Override
    public Result edit(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String id = paramsMap.get("id");
        String stateType = paramsMap.get("stateType");
        //拒绝
        if (type == "1") {
            String refuseReason = paramsMap.get("refuseReason"); //拒绝理由
            if (refuseReason == "" || refuseReason == null) {
                return new Result(ResultCode.FAIL, "拒绝理由不能为空！");
            }
            QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("id", id);
            BillSubsidy billSubsidy = subsidyMapper.selectOne(queryWrapper);
            billSubsidy.setSubsidySch(2);
            billSubsidy.setAcceptTime(new Date());
            subsidyMapper.insert(billSubsidy);
            return new Result(ResultCode.SUCCESS, "更新状态成功！");
        } else { //受理
            QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("id", id);
            BillSubsidy billSubsidy = subsidyMapper.selectOne(queryWrapper);
            if (stateType == "1") { //驳回
                String rejectReason = paramsMap.get("rejectReason");
                String workAccptUrl = paramsMap.get("workAccptUrl");
                if (rejectReason == null || rejectReason == "") {
                    return new Result(ResultCode.FAIL, "驳回理由不能为空！");
                }
                if (workAccptUrl == null || workAccptUrl == "") {
                    return new Result(ResultCode.FAIL, "工作验收表不能为空！");
                }
                billSubsidy.setSubsidySch(4);
                billSubsidy.setRejectReason(rejectReason);
                billSubsidy.setWorkAccptUrl(workAccptUrl);
                billSubsidy.setAcceptTime(new Date());
                subsidyMapper.insert(billSubsidy);
                return new Result(ResultCode.SUCCESS, "更新状态成功！");
            } else { //通过
                billSubsidy.setSubsidySch(3);
                billSubsidy.setAcceptTime(new Date());
                String workAccptUrl = paramsMap.get("workAccptUrl");
                if (workAccptUrl == null || workAccptUrl == "") {
                    return new Result(ResultCode.FAIL, "工作验收表不能为空！");
                }
                billSubsidy.setWorkAccptUrl(workAccptUrl);
                billSubsidy.setReviewedTime(new Date());
                subsidyMapper.insert(billSubsidy);
                return new Result(ResultCode.SUCCESS, "更新状态成功！");
            }
        }
    }

    @Override
    public Result appeal(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        Integer type = Integer.parseInt(paramsMap.get("type"));
        QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id);
        BillSubsidy billSubsidy = subsidyMapper.selectOne(queryWrapper);
        if (type == 1) { //驳回
            String appeal = paramsMap.get("appeal");
            billSubsidy.setSubsidySch(4);
            billSubsidy.setAppealTime(new Date());
            billSubsidy.setAppeal(appeal);
            subsidyMapper.insert(billSubsidy);
        } else {
            String appeal = paramsMap.get("appeal");
            billSubsidy.setSubsidySch(3);
            billSubsidy.setAppealTime(new Date());
            billSubsidy.setAppeal(appeal);
            subsidyMapper.insert(billSubsidy);
        }
        return new Result(ResultCode.SUCCESS, "更新状态成功！");
    }
    @Resource
    private BaseAnimalMapper baseAnimalMapper;

    @Override
    @Transactional
    public Result addApplication(HttpServletRequest httpServletRequest, BillSubsidy billSubsidy) {
        String eartagNo = billSubsidy.getEartagNo();
        String[] eartags = eartagNo.split(";");
        for (String eartag : eartags) {
            BaseAnimal animal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", eartag));
            animal.setIsLoated(1);
            baseAnimalMapper.updateById(animal);
        }
        billSubsidy.setSubsidyName(TokenSign.getUserName(httpServletRequest)+"的补贴申请");
        billSubsidy.setCreateTime(DateTime.now());
        billSubsidy.setCreateUser(Long.valueOf(TokenSign.getUserName(httpServletRequest)));
        if (this.saveOrUpdate(billSubsidy)) {
            return Result.SUCCESS();
        }else {
            return Result.FAIL();
        }
    }

    @Override
    public Result saveApplication(HttpServletRequest httpServletRequest, BillSubsidy billSubsidy) {
        billSubsidy.setCreateUser(Long.valueOf(TokenSign.getUserName(httpServletRequest)));
        if (this.save(billSubsidy)) {
            return Result.SUCCESS();
        }else {
            return Result.FAIL();
        }
    }
}
